@@ -1,38 +1,21 @@
 """The main file."""
 import json
-import re
-import webbrowser
-from datetime import datetime, timezone
-from typing import List
+from datetime import datetime
 from urllib.parse import quote
 
-import bs4
 import humanize
-import questionary
 import requests
 import rich
 import typer
-from packaging.version import parse as parse_version
 from rich.console import Console
-from rich.markdown import Markdown
 from rich.panel import Panel
 from rich.table import Table
-from rich.text import Text
 from rich.theme import Theme
-from rich_rst import RestructuredText
 from typer import Argument, Option
 
 # We instantiate a typer app and a rich console for later use
 app = typer.Typer(add_completion=False)
 console = Console(theme=Theme({"markdown.link": "#6088ff"}))
-# We will use this styling in the browse command
-link_style = questionary.Style(
-    [
-        ("name", "bold red"),
-        ("seperator", "gray"),
-        ("url", "cyan"),
-    ]
-)
 # We will use these headers for every request
 headers = {"User-Agent": "wasi_master/pypi_cli"}
 
@@ -40,7 +23,7 @@ headers = {"User-Agent": "wasi_master/pypi_cli"}
 class Package:
     """Represents a package gotten from scraping the search results."""
 
-    def __init__(self, soup: bs4.BeautifulSoup):
+    def __init__(self, soup):
         """Instantiates a package
 
         Parameters
@@ -114,6 +97,8 @@ def _format_xml_packages(url, title, pubmsg, _author, _link, *, split_title=Fals
     with console.status("Fetching packages"):
         response = requests.get(url, headers=headers)
     soup = bs4.BeautifulSoup(response.text, "lxml-xml")
+    from datetime import timezone  # pylint: disable=import-outside-toplevel
+
     for index, package in enumerate(soup.find_all("item"), 1):
         title = package.find("title").text
         if split_title:
@@ -121,6 +106,7 @@ def _format_xml_packages(url, title, pubmsg, _author, _link, *, split_title=Fals
         author = package.find("author")
         description = package.find("description")
         link = package.find("link").text
+
         date = utc_to_local(datetime.strptime(package.find("pubDate").text, "%a, %d %b %Y %H:%M:%S GMT")).replace(
             tzinfo=None
         )
@@ -162,6 +148,8 @@ def desc(
         response = requests.get(url, headers=headers)
     parsed_data = json.loads(response.text)["info"]
     if force_github:
+        import re  # pylint: disable=import-outside-toplevel
+
         repo = re.findall(r"https://(www\.)?github\.com/([A-Za-z0-9_.-]{0,38}/[A-Za-z0-9_.-]{0,100})", str(parsed_data))
         repo = repo[0][1] if repo else None
         if not repo:
@@ -180,6 +168,8 @@ def desc(
             parsed_data["description_content_type"] = "text/markdown"
     if not parsed_data["description"] or parsed_data["description"] == "UNKNOWN":
         console.print("[red]No description found on PyPI.[/]")
+        import re  # pylint: disable=import-outside-toplevel
+
         repo = re.findall(r"https://(www\.)?github\.com/([A-Za-z0-9_.-]{0,38}/[A-Za-z0-9_.-]{0,100})", str(parsed_data))
         repo = repo[0][1] if repo else None
         if repo:
@@ -203,10 +193,16 @@ def desc(
                 raise typer.Exit()
 
     if parsed_data["description_content_type"] == "text/markdown":
+        from rich.markdown import Markdown  # pylint: disable=import-outside-toplevel
+
         description = Markdown(parsed_data["description"])
     elif parsed_data["description_content_type"] == "text/x-rst":
+        from rich_rst import RestructuredText  # pylint: disable=import-outside-toplevel
+
         description = RestructuredText(parsed_data["description"])
     else:
+        from rich.text import Text  # pylint: disable=import-outside-toplevel
+
         description = Text(parsed_data["description"])
     console.print(Panel(description, title=f"Description for {package_name}", border_style="bold magenta"))
 
@@ -217,6 +213,8 @@ def new_packages(
     _link: bool = Option(True, metavar="link", help="Show the project link of not"),
 ):
     """See the top 40 newly added packages."""
+    import bs4  # pylint: disable=import-outside-toplevel
+
     _format_xml_packages(
         "https://pypi.org/rss/packages.xml",
         "Newly Added Packages",
@@ -233,6 +231,8 @@ def new_releases(
     _link: bool = Option(True, metavar="link", help="Show the project link of not"),
 ):
     """See the top 100 newly updated packages."""
+    import bs4  # pylint: disable=import-outside-toplevel
+
     _format_xml_packages(
         "https://pypi.org/rss/updates.xml",
         "Newly Released Packages",
@@ -281,9 +281,9 @@ def largest_files():
 def search(
     name: str = Argument(..., help="The name of the package to search for"),
     page: int = Option(1, min=1, max=500, help="The page of the search results to show."),
-    classifier: List[str] = Option(
-        None, help="Can be used multiple times to specify a list of classifiers to filter the results."
-    ),
+    # classifier: List[str] = Option(
+    #     None, help="Can be used multiple times to specify a list of classifiers to filter the results."
+    # ),
 ):
     """Search for a package on PyPI."""
     url = "https://pypi.org/search/"
@@ -291,16 +291,18 @@ def search(
         "q": name,
         "page": page,
     }
-    if classifier:
-        parameters["c"] = classifier
+    # if classifier:
+    #     parameters["c"] = classifier
     with console.status(f"Searching for {name}..."):
         response = requests.get(url, headers=headers, params=parameters)
 
     if response.status_code == 404:
-        console.print(f"[bold]The specified page doesn't exist[/]")
+        console.print("[bold]The specified page doesn't exist[/]")
         raise typer.Exit()
 
     with console.status("Parsing data..."):
+        import bs4  # pylint: disable=import-outside-toplevel
+
         soup = bs4.BeautifulSoup(response.text, "lxml")
         result_list = soup.find(attrs={"aria-label": "Search results"}, class_="unstyled")
         if not result_list:
@@ -417,6 +419,8 @@ def info(
     releases = parsed_data["releases"]
     urls = parsed_data["urls"]
 
+    from packaging.version import parse as parse_version  # pylint:disable=import-outside-toplevel
+
     # HACK: should use fromisotime
     release_time = datetime.strptime(urls[-1]["upload_time_iso_8601"], "%Y-%m-%dT%H:%M:%S.%fZ")
     natural_time = release_time.strftime("%b %d, %Y")
@@ -428,12 +432,16 @@ def info(
         if str(latest_version) == str(info["version"])
         else f"[red]Newer version available ({latest_version})[/]"
     )
+    import re  # pylint: disable=import-outside-toplevel
+
     repo = re.findall(
         r"https://(www\.)?github\.com/([A-Za-z0-9_.-]{0,38}/[A-Za-z0-9_.-]{0,100})(?:\.git)?",
         str(info),
     )
     repo = remove_dot_git(repo[0][1]) if repo else None
 
+    from rich.text import Text  # pylint:disable=import-outside-toplevel
+
     title = Text.from_markup(f"[bold cyan]{info['name']} {info['version']}[/]\n{description}", justify="left")
     message = Text.from_markup(f"{version_comment}\nReleased: {natural_time}", justify="right")
     table = Table.grid(expand=True)
@@ -563,6 +571,8 @@ def regex_search(
     all_packages_url = "https://pypi.org/simple/"
     with console.status("Fetching packages"):
         response = requests.get(all_packages_url)
+    import re  # pylint: disable=import-outside-toplevel
+
     packages = re.findall(r"<a[^>]*>([^<]+)<\/a>", response.text)
     # We compile the regex because it's twice as fast (https://imgur.com/a/MoUyEMg)
     _regex = re.compile(regex)
@@ -591,9 +601,114 @@ def regex_search(
             console.print("[yellow]There are more than 50 matches, consider using the --compact flag")
 
 
+@app.command()
+def rtfd(
+    package_name: str = Argument(..., help="The name or link to the docs of the package to show the documentation for"),
+    query: str = Argument(..., help="The query you want to read the docs for"),
+):
+    """Search the documentation for an item of a package."""
+    import webbrowser  # pylint: disable=import-outside-toplevel
+
+    docs_mapping = {
+        **dict.fromkeys(["py", "python", "python3"], "https://docs.python.org/3/search.html"),
+        **dict.fromkeys(["py2", "python2"], "https://docs.python.org/2/search.html"),
+        **dict.fromkeys(["pil", "pillow"], "https://pillow.readthedocs.io/en/stable/search.html"),
+        "aiohttp": "https://docs.aiohttp.org/en/stable/search.html",
+        "attrs": "https://www.attrs.org/en/stable/search.html",
+        "babel": "https://babel.readthedocs.io/en/latest/search.html",
+        "boto3": "https://boto3.amazonaws.com/v1/documentation/api/latest/search.html",
+        "cachetools": "https://cachetools.readthedocs.io/en/latest/search.html",
+        "cffi": "https://cffi.readthedocs.io/en/latest/search.html",
+        "chardet": "https://chardet.readthedocs.io/en/latest/search.html",
+        "cryptography": "https://cryptography.io/en/latest/search.html",
+        "cv2": "http://docs.opencv.org/2.4/search.html",
+        "discord.py": "https://discordpy.readthedocs.io/en/latest/search.html",
+        "django": "http://docs.djangoproject.com/en/dev/search",
+        "dnspython": "https://dnspython.readthedocs.io/en/latest/search.html",
+        "flask": "https://flask.palletsprojects.com/en/1.1.x/search",
+        "h5py": "http://docs.h5py.org/en/latest/search.html",
+        "importlib-metadata": "https://importlib-metadata.readthedocs.io/en/latest/search.html",
+        "importlib-resources": "https://importlib-resources.readthedocs.io/en/latest/search.html",
+        "importlib_metadata": "https://importlib-metadata.readthedocs.io/en/latest/search.html",
+        "importlib_resources": "https://importlib-resources.readthedocs.io/en/latest/search.html",
+        "matplotlib": "https://matplotlib.org/stable/search.html",
+        "natsort": "https://natsort.readthedocs.io/en/master/search.html",
+        "numpy": "http://docs.scipy.org/doc/numpy/search.html",
+        "oauthlib": "https://oauthlib.readthedocs.io/en/latest/search.html",
+        "packaging": "https://packaging.pypa.io/en/latest/search.html",
+        "pandas": "https://pandas.pydata.org/docs/search.html",
+        "psutil": "https://psutil.readthedocs.io/en/latest/search.html",
+        "pydash": "https://pydash.readthedocs.io/en/latest/search.html",
+        "pyjwt": "https://pyjwt.readthedocs.io/en/latest/search.html",
+        "pyopenssl": "https://www.pyopenssl.org/en/latest/search.html",
+        "pyparsing": "https://pyparsing-docs.readthedocs.io/en/latest/search.html",
+        "pyqt": "https://doc.qt.io/qtforpython/search.html",
+        "pyramid": "https://docs.pylonsproject.org/projects/pyramid/en/latest/search.html",
+        "pyrsistent": "https://pyrsistent.readthedocs.io/en/latest/search.html",
+        "pytest": "https://docs.pytest.org/en/stable/search.html",
+        "pytest-regressions": "https://pytest-regressions.readthedocs.io/en/latest/search.html",
+        "python-dateutil": "https://dateutil.readthedocs.io/en/stable/search.html",
+        "pytorch": "https://pytorch.org/docs/stable/search.html",
+        "requests": "https://requests.readthedocs.io/en/master/search.html",
+        "requests-oauthlib": "https://requests-oauthlib.readthedocs.io/en/latest/search.html",
+        "scikit-learn": "https://scikit-learn.org/stable/search.html",
+        "scipy": "https://docs.scipy.org/doc/scipy/search.html",
+        "six": "https://six.readthedocs.io/search.html",
+        "slumber": "https://slumber.readthedocs.io/en/v0.6.0/search.html",
+        "sphinx": "https://www.sphinx-doc.org/en/master/search.html",
+        "yarl": "https://yarl.readthedocs.io/en/latest/search.html",
+        "zipp": "https://zipp.readthedocs.io/en/latest/search.html",
+    }
+
+    if package_name[:4] == "http":
+        url = package_name
+    else:
+        if package_name in docs_mapping:
+            url = docs_mapping[package_name]
+        else:
+            import questionary  # pylint: disable=import-outside-toplevel
+
+            resp = questionary.confirm("Docs not available.Do you want to search pypi to find the documentation?").ask()
+            if resp:
+                url = f"https://pypi.org/pypi/{quote(package_name)}/json"
+                with console.status("Getting data from PyPI"):
+                    response = requests.get(url, headers=headers)
+
+                if response.status_code != 200:
+                    if response.status_code == 404:
+                        rich.print("[red]Project not found[/]")
+                    rich.print(f"[orange]Some error occured. response code {response.status_code}[/]")
+                    raise typer.Exit()
+
+                parsed_data = json.loads(response.text)
+                url = parsed_data["info"].get("project_urls", {}).get("Documentation", None)
+                if not url:
+                    console.print("[bold]Documentation url not found on PyPI[/]")
+                    raise typer.Exit()
+                else:
+                    url = url + "/search.html"
+            else:
+                console.print("[dim grey]Cancelled![/]")
+                raise typer.Exit()
+
+    search_page = url + "?q=" + quote(query)
+    webbrowser.open(search_page)
+
+
 @app.command()
 def browse(package_name: str = Argument(...)):
     """Browse for a package's URLs"""
+    import questionary  # pylint: disable=import-outside-toplevel
+    import webbrowser  # pylint: disable=import-outside-toplevel
+
+    link_style = questionary.Style(
+        [
+            ("name", "bold red"),
+            ("seperator", "gray"),
+            ("url", "cyan"),
+        ]
+    )
+
     url = f"https://pypi.org/pypi/{quote(package_name)}/json"
     with console.status("Getting data from PyPI"):
         response = requests.get(url, headers=headers)
@@ -616,7 +731,9 @@ def browse(package_name: str = Argument(...)):
     answer = questionary.select(
         "Which link do you want to to open?",
         choices=[
-            questionary.Choice(title=[("class:name", name), ("class:seperator", " - "), ("class:url", url)], value=url)
+            questionary.Choice(
+                title=[("class:name", f"{name:15}"), ("class:seperator", " - "), ("class:url", url)], value=url
+            )
             for name, url in urls.items()
             if url
         ],
